##################################################
#Desc:   Wolf SSL lib generator
#Author: Mihrtec LLC
#Date:   2016
##################################################
#Included from Makefile
#Defines rules for automatically downloading,
#compiling, and linking the wolfssl library
##################################################

#wolfssl module compilation rules
WOLFDIR  := $(MLSRCDIR)/wolfssl
WOLFSO   := $(WOLFDIR)/src/.libs/libwolfssl.so
WOLFTARG := $(MLLIBDIR)/libwolfssl.so
CONFCL   := --enable-singlethreaded=yes
CONFCL   += --enable-static=yes
CONFCL   += --enable-fastmath=no
CONFCL   += --enable-cryptonly=yes
CONFCL   += CFLAGS=-w
GITADDR  := git@github.com:wolfSSL/wolfssl.git

#Remove the WOLFSO after compile since it is
#compiler-dependent, and we copy it to WOLFTARG
#which is a compiler-dependent location
.INTERMEDIATE: $(WOLFSO)

#WOLFTARG is the target file to generate for the
#missing library.  It depends on WOLFSO, an
#intermediate file generated only when we need
#to copy it to WOLFTARG
$(WOLFTARG): $(WOLFSO)
	@mkdir -p $(dir $(WOLFTARG))
	@cp -H $(WOLFSO) $(WOLFTARG)
	@echo "WolfSSL library created"

$(WOLFSO): $(WOLFDIR)
	@cd $(WOLFDIR) && $(AMGEN) && $(AMCONF) $(CONFCL)
	@sed -i 's@SIZEOF_LONG 0@SIZEOF_LONG 4@' $(WOLFDIR)/config.h
	@sed -i 's@SIZEOF_LONG_LONG 0@SIZEOF_LONG_LONG 8@' $(WOLFDIR)/config.h
	@cd $(WOLFDIR) && export CFLAGS=-w && $(AMMAKE) src/libwolfssl.la

$(WOLFDIR):
	@mkdir -p $(WOLFDIR)
	@git clone $(GITADDR) $(WOLFDIR)

clean_wolfssl:
	cd $(WOLFDIR) && make clean
